# 5801.py
# ref: https://ko.wikipedia.org/wiki/경사_하강법
from turtle import *
#1
def f(x):
    return x**4-3*x**3 + 2

def fprime(x):
    return 4*x**3-9*x**2

def diff(f, x):
    h = 0.001
    return (f(x+h) - f(x))/h

#2: 경사 하강법
def gradient_descent(f, x_init=4, alpha=0.001, tol=1e-5, maxiter=10000):
  
    x_cur = x_init  # 초기값
    x_lists = [x_cur]
    count = 0
    while count < maxiter:
        #x_new = x_cur - alpha*fprime(x_cur)   # fprime
        x_new = x_cur - alpha*diff(f, x_cur)   # diff
        
        if abs(x_new - x_cur) < tol:
            break
        x_lists.append(x_new) 
        x_cur = x_new
        count += 1
    return x_lists

#3: drawing
#3-1
def line(x1, y1, x2, y2, width=3, color='black'):
    g_pen.pensize(width)
    g_pen.pencolor(color)    
    g_pen.up();   g_pen.setpos(x1, y1)
    g_pen.down(); g_pen.setpos(x2, y2)
    
#3-2
def draw_solution_lists(x_lists):
    g_pen.up()
    #tracer(0)
    for x in x_lists:
        y = f(x) 
        g_pen.setpos(x, y)
        g_pen.dot(5, 'red')
    #update() # tracer(1)
        
#3-3    
def plot(f, min_x = -4, max_x=4, min_y=4, max_y=4, dx = 0.1,
         width=3, color='blue'):
    g_pen.pensize(width)
    g_pen.pencolor(color)
    g_pen.hideturtle()
    x = min_x
    pen_down= False
    while x <= max_x:
        y = f(x)
        
        if not (min_y< y <max_y):
            x += dx*10 # skip out of scope
            continue
        elif not pen_down:
            #print('pen down!!!!!')
            g_pen.up()
            g_pen.setpos(x, y)
            g_pen.write(f'({x}, {y})', font=('Times', 16))
            g_pen.down()
            pen_down=True
        else:
            #print(f'x={x}, y={y}')
            g_pen.setpos(x, y)
        x += dx
    g_pen.write(f'({x:.2f}, {y:.2f})', font=('Times', 16))

#4
def main(x_init=4):
#4-1
    X = gradient_descent(f, x_init)
    x_new = X[-1] # final solution
    print(f'{len(X)} iteration, minimum value: f({x_new}) = {f(x_new)}')

#4-2
    global g_pen
    g_pen= Turtle()
    xmin, xmax = -10, 10
    ymin, ymax = -10, 10
    setworldcoordinates(xmin, ymin, xmax, ymax)

#4-3: draw axis
    tracer(0)
    line(xmin, 0,  xmax-1, 0); g_pen.write('X', font=('Times', 16))
    line(0, ymin,  0, ymax-1); g_pen.write('Y', font=('Times', 16))
    tracer(1)

#4-4: plot f(x)
    plot(f, xmin, xmax, ymin, ymax)

#4-5
    X = list(filter(lambda x: ymin < f(x) <ymax, X))
    tracer(0)
    draw_solution_lists(X)          # 중간해 표시
    tracer(1)
    
    line(x_new, 0, x_new, f(x_new), color='green') # 최소값 표시

    g_pen.up(); g_pen.fd(0.5)
    g_pen.write(f'({x_new:.2f}, {f(x_new):.2f})', font=('Times', 16))

if __name__ == '__main__':
    #main() # x_init=4
    main(x_init=-2)
    mainloop()
