# 6001.py
from turtle import *
from functools import partial

def MSE(a, b, points): # mean-squared error: a:기울기, b: y-절편  
  X, Y = zip(*points)
  eSum=sum(map(lambda x, y: ((a *x + b) - y)**2, X, Y))
  N  = len(points)
  return eSum/N

def line_fit_gradient_descent(points, lr= 0.01, max_epoch=1000, init=[0.1, 0.0]):
  
  N = len(points)
  a, b = init
  X, Y = zip(*points)  # X: xi, Y: yi 
  loss_list = []
  for epoch in range(max_epoch):
    da = sum(map(lambda x, y: ((a*x + b)-y)*x , X, Y))*2/N
    db = sum(map(lambda x, y:  (a*x + b)-y,     X, Y))*2/N
    
    a = a - lr*da
    b = b - lr*db

    loss = MSE(a, b, points)
    loss_list.append(loss)
    
  return a, b, loss_list

def line(x1, y1, x2, y2, width=3, color='black'):
    g_pen.hideturtle()
    g_pen.pensize(width)
    g_pen.pencolor(color)    
    g_pen.up();   g_pen.setpos(x1, y1)
    g_pen.down(); g_pen.setpos(x2, y2)

def draw_points(points, color='red'):
    g_pen.hideturtle()
    g_pen.up()
    for x, y in points:
        g_pen.setpos(x, y)
        g_pen.dot(10, color)

def draw_losses(losses, color='red'):
    screen = Screen()
    screen.clear()

    N = len(losses)
    STEP = N//10
    
    xmin, xmax = 0, len(losses)
    ymin, ymax = 0, max(losses)
    print(f'xmin={xmin}, xmax={xmax}, ymin={ymin}, ymax={ymax}')
    setworldcoordinates(xmin-STEP, ymin-1, xmax+STEP, ymax+1)
    
    tracer(0)
    # draw axis 
    line(xmin, 0,  xmax, 0)
    g_pen.up();g_pen.setpos(xmax/2, -0.4)
    g_pen.write('epoch', font=('Times', 14))
 
    for x in range(0, N+1, STEP):
      g_pen.setpos(x, -0.2)
      g_pen.write(f'{x}', font=('Times', 14))          

    line(0, ymin, 0, ymax-1)
    g_pen.setpos(0,losses[0]) # start point
    g_pen.write(f'loss={losses[0]:.4f}', font=('Times', 14))
    #tracer(1)

    #draw losses
    g_pen.pencolor(color)
    g_pen.setpos(0,losses[0])
    g_pen.down() 
    for i, loss in enumerate(losses):        
      g_pen.setpos(i, loss)
      g_pen.dot(1, color)
    g_pen.write(f'{losses[-1]:.4f}', font=('Times', 14))
    update()
   
def main(points):
    global g_pen
    g_pen = Turtle()
    setworldcoordinates(-1, -1, 6, 6)
    
    tracer(0)
    xmin, xmax = -1, 5
    ymin, ymax = -1, 5
    line(xmin, 0,  xmax, 0); g_pen.write('X', font=('Times', 14))
    line(0, ymin,  0, ymax); g_pen.write('Y', font=('Times', 14))
   
    draw_points(points)
    a, b, losses = line_fit_gradient_descent(points)
    error = losses[-1]
    print(f'y = {a}x + {b}')
    print(f'MSE(a, b)= {error}')
        
    x1 = xmin
    y1 = a*x1 + b
    x2 = xmax
    y2 = a*x2 + b
    line(x1, y1, x2, y2, color='blue')
    g_pen.write(f'y = {a:.2f}x + {b:.2f}',font=('Times', 16)) 
    update() #tracer(1)


    onkey(Screen().bye,'Escape')
    onkey(partial(draw_losses,losses), 'Return')   
    listen()
       
if __name__ == '__main__':
    points = [(1, 2), (2, 3), (3, 4), (4, 4)]
    main(points)
    mainloop() 
